/*
 * Programmieren fuer Physiker, SS 2023
 *
 * Tuerme von Hanoi, lange Loesung (mit Ausgabe der Stangen)
 */

#include <iostream>
#include <iomanip>
#include <cstdlib>
using namespace std ;

const int maxScheiben=8 ;

// ---------------------------------------------------------------------

class ClTurm
{
private:
    int AnzSch ;  // Anzahl der Scheiben des Turms
    int SRad[maxScheiben] ;  // die Scheiben selbst, Index 0: unterste
public:
    ClTurm() ; // Konstruktor fuer leeren Turm
    int Nimm_Oberste() ;
    void Lege_Obenauf(int rad) ;
    int Gib_Radius_In_Hoehe(int h) ;
} ;

ClTurm::ClTurm() 
{
    AnzSch = 0 ;
    // cout << "Konstruktor aufgerufen..." << endl ;
}

int ClTurm::Nimm_Oberste() 
{
    if (AnzSch<=0) { cout << "Huch. Turm schon leer..." ; exit (1) ; } ;
    AnzSch-- ;
    return SRad[AnzSch] ;
}
    
void ClTurm::Lege_Obenauf(int rad) 
{
    if (AnzSch>=maxScheiben) 
    { cout << "Huch. Turm schon voll..." ; exit (1) ; } ;
    // cout << "Hoehe: " << AnzSch << "  Scheibe: " << rad << endl ;
    // hier moeglich: Test, ob legaler Zug! (Scheibengroesse)
    SRad[AnzSch] = rad ;
    AnzSch++ ;
}
    
int ClTurm::Gib_Radius_In_Hoehe(int h) 
{
    if (h>=AnzSch) return 0 ;  // Radius 0 : keine Scheibe!
    else return SRad[h] ;
}

// ---------------------------------------------------------------------

class ClSpielbrett
{
private:
    ClTurm Turm[3] ;  // 3 Tuerme
public:
    ClSpielbrett( int n) ;
    void Zeichne() ;
    void BewegeOberste(int qu, int zl) ;
} ;

// ---------------------------------------------------------------------

ClSpielbrett::ClSpielbrett( int n) 
{
    // Konstruktoren jedes Einzelturms werden aufgerufen -> alle leer
    for( int i=n; i>=1; i--)  // baue Startposition auf
	Turm[0].Lege_Obenauf(i) ;
}

void ZeichneScheibe(int s) 
{
    // s>=1: Scheibe Groesse s, s==0: leer
    if (s==0)
	cout << setw(11) << "|" << setw(10) << " " ;
    else
	cout << setw(11-s) << "[" 
	     << setw(2*s) << setfill('#') << "]"
	     << setw(10-s) << setfill(' ') << " " ;
}

void ClSpielbrett::Zeichne()
{
    for(int i=maxScheiben-1; i>=0; i--)
    {
	for(int st=0; st<=2; st++)
	    ZeichneScheibe( Turm[st].Gib_Radius_In_Hoehe(i)) ;
	cout << endl ;
    }
    cout << setw(3*21) << setfill('=') << "=" << setfill(' ') << endl ;
}

void ClSpielbrett::BewegeOberste( int qu, int zl)
{
    if (qu<0 or qu>2 or zl<0 or zl>2)
    { cout << "Bewege: Stangenindex ist nicht ok!" ; exit (1) ; } ;
    if (qu==zl)
    { cout << "Bewege: Quellstange==Zielstange?" ; return ; } ;
    Turm[zl].Lege_Obenauf (Turm[qu].Nimm_Oberste()) ;
}
// ---------------------------------------------------------------------

void Hanoi( int n, int quelle, int ziel, int hilf, ClSpielbrett & Spiel) 
{
    // Lege n Scheiben von Stange 'quelle' nach Stange 'ziel'
    // unter Verwendung der Hilfsstange 'hilf'
    // Stangen haben int-nummern 0 bis 2
    if( n==0) return ; // nichts zu tun.

    // Vorgehen: n-1 Scheiben von 'quelle' nach 'hilf' bringen
    //           Scheibe n von 'quelle' nach 'ziel' legen
    //           die n-1 von 'hilf' nach 'ziel'
    Hanoi( n-1, quelle, hilf, ziel, Spiel) ;

    cout << "Lege Scheibe " << n << " von " << quelle
	 << " nach " << ziel << endl ; 
    Spiel.BewegeOberste( quelle, ziel) ;
    Spiel.Zeichne() ;
    cin.get() ; // wait for return key

    Hanoi( n-1, hilf, ziel, quelle, Spiel) ;
}

int main()
{
    int n ;
    cout << "Tuerme von Hanoi" << endl ;
    cout << "Wieviel Scheiben: " ;
    cin >> n ;
    cin.ignore() ; // leere Eingabepuffer

    ClSpielbrett Spiel(n) ;
    Spiel.Zeichne() ;
    cin.get() ; // wait for return key

    Hanoi( n, 0, 1, 2, Spiel) ;
}
